import datetime
import random

import requests
import requests_cache
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from modelcluster.contrib.taggit import ClusterTaggableManager
from modelcluster.fields import ParentalKey
from taggit.models import TaggedItemBase
from wagtail.admin.edit_handlers import FieldPanel, HelpPanel
from wagtail.core.models import Page
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtailmetadata.models import MetadataPageMixin

# cacheovani requestu na piratska api, ttl 1 hodina
requests_cache.install_cache(
    cache_name="shared_models", backend="memory", expire_after=3600
)


class SharedSubpageMixin:
    """Must be used in class definition before MetadataPageMixin!"""

    @property
    def root_page(self):
        if not hasattr(self, "_root_page"):
            # Vypada to hackove ale lze takto pouzit: dle dokumentace get_ancestors vraci stranky v poradi
            # od rootu, tedy domovska stranka je druha v poradi
            self._root_page = self.get_ancestors().specific()[1]
        return self._root_page


class ArticleTag(TaggedItemBase):
    content_object = ParentalKey("shared.Article", on_delete=models.CASCADE)


class ArticleMixin(models.Model):
    """ Spolecna pole vsech clanku """

    perex = models.CharField("perex", max_length=250, blank=True)
    text = models.TextField("text článku", blank=True)
    date = models.DateField("datum článku", blank=False, default=datetime.datetime.now)

    tags = ClusterTaggableManager(through=ArticleTag, blank=True)
    image = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name="obrázek",
    )
    author = models.ForeignKey(
        "shared.PersonPage",
        verbose_name="Autor článku",
        on_delete=models.PROTECT,
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True
        verbose_name = "Článek"


class Article(ArticleMixin, Page, SharedSubpageMixin, MetadataPageMixin):
    """ Univerzalni clanek pro vsechny weby """

    content_panels = Page.content_panels + [
        FieldPanel("perex"),
        FieldPanel("date"),
        FieldPanel("text"),
        HelpPanel(
            """Text článku lze formátovat s použitím <a href="https://www.markdownguide.org/basic-syntax/">markdown</a>"""
        ),
        FieldPanel("author"),
        ImageChooserPanel("image"),
        FieldPanel("tags"),
    ]


class PersonPage(SharedSubpageMixin, MetadataPageMixin, Page):
    """ Stranka Pirata.
        Snaha je zadavat tady minimum informaci o osobe a zbytek vytahovat via API z jinych zdroju,
        neb nechceme dalsi databazi lidi.
    """

    # ve formatu iapi: https://iapi.pirati.cz/v1/user?username=john.doe
    username = models.TextField("Pirate ID osoby", blank=False)
    profile_id = models.IntegerField(
        "Číslo medailonku (0..2) z lide.pirati.cz",
        blank=True,
        default=0,
        validators=[MinValueValidator(0), MaxValueValidator(2)],
    )

    perex = models.TextField("Perex osoby", blank=True)

    content_panels = Page.content_panels + [
        FieldPanel("username"),
        FieldPanel("profile_id"),
        FieldPanel("perex"),
    ]

    def _iapi(self):
        """ Vrati data o osobe z piratskeho IAPI jako json. Cached via requests_cache """
        return requests.get(
            "https://iapi.pirati.cz/v1/user?username=%s" % self.username
        ).json()

    @property
    def name(self):
        return self._iapi().get("displayname")

    @property
    def email(self):
        try:
            return self._iapi()["profiles"][self.profile_id]["email"]
        except (IndexError, KeyError):
            return None

    @property
    def phone(self):
        try:
            return self._iapi()["profiles"][self.profile_id]["phone"]
        except (IndexError, KeyError):
            return None

    @property
    def longtext(self):  # medailonek
        return ""  # TODO

    @property
    def portrait(self):  # profilova fotka
        try:
            return self._iapi()["profiles"][self.profile_id]["photo"]
        except (IndexError, KeyError):
            return None

    @property
    def facebook(self):
        return None  # TODO

    class Meta:
        verbose_name = "Člověk"  # to zni hrde

    def get_context(self, request):
        context = super().get_context(request)
        # Na strance detailu cloveka se vpravo zobrazuji 3 dalsi nahodne profily
        context["random_people"] = list(
            self.get_siblings(inclusive=False).type(PersonPage).live().specific()
        )
        random.shuffle(context["random_people"])
        context["random_people"] = context["random_people"][:3]
        return context


class PeoplePage(SharedSubpageMixin, MetadataPageMixin, Page):
    """ Stranka se zobrazenim lidi v dane organizaci """

    settings_panels = []

    subpage_types = [
        "shared.PersonPage",
    ]

    class Meta:
        verbose_name = "Stránka Lidé"

    def get_context(self, request):
        context = super().get_context(request)
        context["people"] = self.get_children().live().specific()
        return context
